@@ -1,7 +1,7 @@
 /*
  * This file is part of OpenModelica.
  *
- * Copyright (c) 1998-2014, Open Source Modelica Consortium (OSMC),
+ * Copyright (c) 1998-CurrentYear, Open Source Modelica Consortium (OSMC),
  * c/o Linköpings universitet, Department of Computer and Information Science,
  * SE-58183 Linköping, Sweden.
  *
@@ -37,6 +37,12 @@
 
 #undef smooth
 
+extern "C" {
+#include "meta/meta_modelica.h"
+#include "omc_config.h"
+#include "gc.h"
+}
+
 #include <QtGlobal>
 #if (QT_VERSION < QT_VERSION_CHECK(4, 6, 0))
 #error "OMEdit requires Qt 4.6.0 or newer"
@@ -45,12 +51,14 @@
 #include <QMainWindow>
 #include <QDialog>
 #include <QProgressBar>
+#include <QMimeData>
 #include <QDomDocument>
 #include <QStackedWidget>
 #include <QActionGroup>
 #include <QToolButton>
 #include <QMdiSubWindow>
 #include <QMdiArea>
+#include <QShortcut>
 
 class OMCProxy;
 class TransformationsWidget;
@@ -63,9 +71,13 @@ class GDBLoggerWidget;
 class DocumentationWidget;
 class PlotWindowContainer;
 class VariablesWidget;
+#if !defined(WITHOUT_OSG)
+class ThreeDViewer;
+#endif
 class BreakpointsWidget;
 class SimulationDialog;
 class TLMCoSimulationDialog;
+class OMSSimulationDialog;
 class ModelWidgetContainer;
 class WelcomePageWidget;
 class InfoBar;
@@ -73,6 +85,11 @@ class AboutOMEditDialog;
 class Label;
 class FileDataNotifier;
 class LibraryTreeItem;
+class GitCommands;
+class CommitChangesDialog;
+class TraceabilityInformationURI;
+class TraceabilityGraphViewWidget;
+class SearchWidget;
 
 class MainWindow : public QMainWindow
 {
@@ -84,11 +101,12 @@ private:
   static MainWindow *mpInstance;
 public:
   static MainWindow *instance(bool debug = false);
-  void setUpMainWindow();
+  void setUpMainWindow(threadData_t *threadData);
   bool isDebug() {return mDebug;}
   OMCProxy* getOMCProxy() {return mpOMCProxy;}
   void setExitApplicationStatus(bool status) {mExitApplicationStatus = status;}
   bool getExitApplicationStatus() {return mExitApplicationStatus;}
+  int getNumberOfProcessors() {return mNumberOfProcessors;}
   LibraryWidget* getLibraryWidget() {return mpLibraryWidget;}
   StackFramesWidget* getStackFramesWidget() {return mpStackFramesWidget;}
   BreakpointsWidget* getBreakpointsWidget() {return mpBreakpointsWidget;}
@@ -101,17 +119,29 @@ public:
   PlotWindowContainer* getPlotWindowContainer() {return mpPlotWindowContainer;}
   VariablesWidget* getVariablesWidget() {return mpVariablesWidget;}
   QDockWidget* getVariablesDockWidget() {return mpVariablesDockWidget;}
+  SearchWidget* getSearchWidget() {return mpSearchWidget;}
+
+#if !defined(WITHOUT_OSG)
+  bool isThreeDViewerInitialized();
+  ThreeDViewer* getThreeDViewer();
+  QDockWidget* getThreeDViewerDockWidget() {return mpThreeDViewerDockWidget;}
+#endif
   SimulationDialog* getSimulationDialog() {return mpSimulationDialog;}
   TLMCoSimulationDialog* getTLMCoSimulationDialog() {return mpTLMCoSimulationDialog;}
+  OMSSimulationDialog* getOMSSimulationDialog() {return mpOMSSimulationDialog;}
   ModelWidgetContainer* getModelWidgetContainer() {return mpModelWidgetContainer;}
   WelcomePageWidget* getWelcomePageWidget() {return mpWelcomePageWidget;}
+  GitCommands* getGitCommands() {return mpGitCommands;}
+  CommitChangesDialog* getCommitChangesDialog() {return mpCommitChangesDialog;}
+  TraceabilityInformationURI* getTraceabilityInformationURI() {return mpTraceabilityInformationURI;}
   QStatusBar* getStatusBar() {return mpStatusBar;}
   QProgressBar* getProgressBar() {return mpProgressBar;}
   void showProgressBar() {mpProgressBar->setVisible(true);}
   void hideProgressBar() {mpProgressBar->setVisible(false);}
-  Label* getPointerXPositionLabel() {return mpPointerXPositionLabel;}
-  Label* getPointerYPositionLabel() {return mpPointerYPositionLabel;}
-  QTabBar* getPerspectiveTabBar() {return mpPerspectiveTabbar;}
+  Label* getPositionLabel() {return mpPositionLabel;}
+  bool isModelingPerspectiveActive();
+  bool isPlottingPerspectiveActive();
+  bool isDebuggingPerspectiveActive();
   QTimer* getAutoSaveTimer() {return mpAutoSaveTimer;}
   QAction* getSaveAction() {return mpSaveAction;}
   QAction* getSaveAsAction() {return mpSaveAsAction;}
@@ -124,6 +154,8 @@ public:
   QAction* getResetZoomAction() {return mpResetZoomAction;}
   QAction* getZoomInAction() {return mpZoomInAction;}
   QAction* getZoomOutAction() {return mpZoomOutAction;}
+  QAction* getCloseAllWindowsAction() {return mpCloseAllWindowsAction;}
+  QAction* getCloseAllWindowsButThisAction() {return mpCloseAllWindowsButThisAction;}
   QAction* getSimulateModelAction() {return mpSimulateModelAction;}
   QAction* getSimulateWithTransformationalDebuggerAction() {return mpSimulateWithTransformationalDebuggerAction;}
   QAction* getSimulateWithAlgorithmicDebuggerAction() {return mpSimulateWithAlgorithmicDebuggerAction;}
@@ -135,6 +167,8 @@ public:
   QAction* getCheckModelAction() {return mpCheckModelAction;}
   QAction* getCheckAllModelsAction() {return mpCheckAllModelsAction;}
   QAction* getExportFMUAction() {return mpExportFMUAction;}
+  QAction* getExportEncryptedPackageAction() {return mpExportEncryptedPackageAction;}
+  QAction* getExportRealonlyPackageAction() {return mpExportReadonlyPackageAction;}
   QAction* getExportXMLAction() {return mpExportXMLAction;}
   QAction* getExportFigaroAction() {return mpExportFigaroAction;}
   QAction* getLineShapeAction() {return mpLineShapeAction;}
@@ -149,18 +183,39 @@ public:
   QAction* getImportFromOMNotebookAction() {return mpImportFromOMNotebookAction;}
   QAction* getImportNgspiceNetlistAction() {return mpImportNgspiceNetlistAction;}
   QAction* getConnectModeAction() {return mpConnectModeAction;}
+  QAction* getTransitionModeAction() {return mpTransitionModeAction;}
   QAction* getReSimulateModelAction() {return mpReSimulateModelAction;}
   QAction* getReSimulateSetupAction() {return mpReSimulateSetupAction;}
   QAction* getSimulationParamsAction() {return mpSimulationParamsAction;}
   QAction* getFetchInterfaceDataAction() {return mpFetchInterfaceDataAction;}
   QAction* getAlignInterfacesAction() {return mpAlignInterfacesAction;}
   QAction* getTLMSimulationAction() {return mpTLMCoSimulationAction;}
+  QAction* getAddSystemAction() {return mpAddSystemAction;}
+  QAction* getAddOrEditIconAction() {return mpAddOrEditIconAction;}
+  QAction* getDeleteIconAction() {return mpDeleteIconAction;}
+  QAction* getAddConnectorAction() {return mpAddConnectorAction;}
+  QAction* getAddBusAction() {return mpAddBusAction;}
+  QAction* getAddTLMBusAction() {return mpAddTLMBusAction;}
+  QAction* getAddSubModelAction() {return mpAddSubModelAction;}
+  QAction* getOMSInstantiateModelAction() {return mpOMSInstantiateModelAction;}
+  QAction* getOMSSimulationSetupAction() {return mpOMSSimulateAction;}
+  QAction* getLogCurrentFileAction() {return mpLogCurrentFileAction;}
+  QAction* getStageCurrentFileForCommitAction() {return mpStageCurrentFileForCommitAction;}
+  QAction* getUnstageCurrentFileFromCommitAction() {return mpUnstageCurrentFileFromCommitAction;}
+  QAction* getCommitFilesAction() {return mpCommitFilesAction;}
+  QAction* getRevertCommitAction() {return mpRevertCommitAction;}
+  QAction* getCleanWorkingDirectoryAction() {return mpCleanWorkingDirectoryAction;}
+  QToolBar* getShapesToolBar() const {return mpShapesToolBar;}
+  QToolBar* getCheckToolBar() const {return mpCheckToolBar;}
+  QToolBar* getSimulationToolBar() const {return mpSimulationToolBar;}
+  QToolBar* getTLMSimulationToolbar() const {return mpTLMSimulationToolbar;}
+  QToolBar* getOMSimulatorToobar() const {return mpOMSimulatorToobar;}
   void addRecentFile(const QString &fileName, const QString &encoding);
   void updateRecentFileActions();
-  void closeEvent(QCloseEvent *event);
+  void closeEvent(QCloseEvent *event) override;
   int askForExit();
   void beforeClosingMainWindow();
-  void openDroppedFile(QDropEvent *event);
+  void openDroppedFile(const QMimeData *pMimeData);
   void openResultFiles(QStringList fileNames);
   void simulate(LibraryTreeItem *pLibraryTreeItem);
   void simulateWithTransformationalDebugger(LibraryTreeItem *pLibraryTreeItem);
@@ -169,13 +224,17 @@ public:
   void simulateWithAnimation(LibraryTreeItem *pLibraryTreeItem);
 #endif
   void simulationSetup(LibraryTreeItem *pLibraryTreeItem);
+  void instantiateOMSModel(LibraryTreeItem *pLibraryTreeItem, bool checked);
+  void simulateOMSModel(LibraryTreeItem *pLibraryTreeItem);
   void instantiateModel(LibraryTreeItem *pLibraryTreeItem);
   void checkModel(LibraryTreeItem *pLibraryTreeItem);
   void checkAllModels(LibraryTreeItem *pLibraryTreeItem);
   void exportModelFMU(LibraryTreeItem *pLibraryTreeItem);
+  void exportEncryptedPackage(LibraryTreeItem *pLibraryTreeItem);
+  void exportReadonlyPackage(LibraryTreeItem *pLibraryTreeItem);
   void exportModelXML(LibraryTreeItem *pLibraryTreeItem);
   void exportModelFigaro(LibraryTreeItem *pLibraryTreeItem);
-  void fetchInterfaceData(LibraryTreeItem *pLibraryTreeItem);
+  void fetchInterfaceData(LibraryTreeItem *pLibraryTreeItem, QString singleModel=QString());
   void TLMSimulate(LibraryTreeItem *pLibraryTreeItem);
   void exportModelToOMNotebook(LibraryTreeItem *pLibraryTreeItem);
   void createOMNotebookTitleCell(LibraryTreeItem *pLibraryTreeItem, QDomDocument xmlDocument, QDomElement domElement);
@@ -183,18 +242,22 @@ public:
   void createOMNotebookCodeCell(LibraryTreeItem *pLibraryTreeItem, QDomDocument xmlDocument, QDomElement domElement);
   TransformationsWidget* showTransformationsWidget(QString fileName);
   void findFileAndGoToLine(QString fileName, QString lineNumber);
+  void printStandardOutAndErrorFilesMessages();
   static void PlotCallbackFunction(void *p, int externalWindow, const char* filename, const char* title, const char* grid,
                                    const char* plotType, const char* logX, const char* logY, const char* xLabel, const char* yLabel,
                                    const char* x1, const char* x2, const char* y1, const char* y2, const char* curveWidth,
                                    const char* curveStyle, const char* legendPosition, const char* footer, const char* autoScale,
                                    const char* variables);
+
+  QList<QString> mFMUDirectoriesList;
 private:
   bool mDebug;
   OMCProxy *mpOMCProxy;
   bool mExitApplicationStatus;
+  int mNumberOfProcessors;
+  SearchWidget *mpSearchWidget;
+  QDockWidget *mpSearchDockWidget;
   QDockWidget *mpMessagesDockWidget;
-  FileDataNotifier *mpOutputFileDataNotifier;
-  FileDataNotifier *mpErrorFileDataNotifier;
   LibraryWidget *mpLibraryWidget;
   QDockWidget *mpLibraryDockWidget;
   GDBAdapter *mpGDBAdapter;
@@ -213,34 +276,59 @@ private:
   PlotWindowContainer *mpPlotWindowContainer;
   VariablesWidget *mpVariablesWidget;
   QDockWidget *mpVariablesDockWidget;
+  TraceabilityGraphViewWidget *mpTraceabilityGraphViewWidget;
+#if !defined(WITHOUT_OSG)
+  ThreeDViewer *mpThreeDViewer;
+  QDockWidget *mpThreeDViewerDockWidget;
+#endif
   SimulationDialog *mpSimulationDialog;
   TLMCoSimulationDialog *mpTLMCoSimulationDialog;
+  OMSSimulationDialog *mpOMSSimulationDialog;
   ModelWidgetContainer *mpModelWidgetContainer;
   WelcomePageWidget *mpWelcomePageWidget;
+  GitCommands *mpGitCommands;
+  CommitChangesDialog *mpCommitChangesDialog;
+  TraceabilityInformationURI *mpTraceabilityInformationURI;
   QStackedWidget *mpCentralStackedWidget;
   QProgressBar *mpProgressBar;
-  Label *mpPointerXPositionLabel;
-  Label *mpPointerYPositionLabel;
+  Label *mpPositionLabel;
   QTabBar *mpPerspectiveTabbar;
   QStatusBar *mpStatusBar;
   QTimer *mpAutoSaveTimer;
+  QShortcut *mpSearchBrowserShortcut;
   // File Menu
   // Modelica File Actions
   QAction *mpNewModelicaClassAction;
   QAction *mpOpenModelicaFileAction;
   QAction *mpOpenModelicaFileWithEncodingAction;
   QAction *mpLoadModelicaLibraryAction;
+  QAction *mpLoadEncryptedLibraryAction;
   QAction *mpOpenResultFileAction;
   QAction *mpOpenTransformationFileAction;
-  // MetaModel File Actions
-  QAction *mpNewMetaModelFileAction;
-  QAction *mpOpenMetaModelFileAction;
+  // CompositeModel File Actions
+  QAction *mpNewCompositeModelFileAction;
+  QAction *mpOpenCompositeModelFileAction;
   QAction *mpLoadExternModelAction;
+  // OMSimulator File Actions
+  QAction *mpNewOMSimulatorModelAction;
+  QAction *mpOpenOMSModelFileAction;
   QAction *mpOpenDirectoryAction;
   QAction *mpSaveAction;
   QAction *mpSaveAsAction;
   QAction *mpSaveAllAction;
   QAction *mpSaveTotalAction;
+  QAction *mpImportFMUAction;
+  QAction *mpImportFMUModelDescriptionAction;
+  QAction *mpImportFromOMNotebookAction;
+  QAction *mpImportNgspiceNetlistAction;
+  QAction *mpExportToClipboardAction;
+  QAction *mpExportAsImageAction;
+  QAction *mpExportFMUAction;
+  QAction *mpExportReadonlyPackageAction;
+  QAction *mpExportEncryptedPackageAction;
+  QAction *mpExportXMLAction;
+  QAction *mpExportFigaroAction;
+  QAction *mpExportToOMNotebookAction;
   QAction *mpRecentFileActions[MaxRecentFiles];
   QAction *mpClearRecentFilesAction;
   QAction *mpPrintModelAction;
@@ -249,9 +337,6 @@ private:
   QAction *mpUndoAction;
   QAction *mpRedoAction;
   QAction *mpFilterClassesAction;
-  QAction *mpCutAction;
-  QAction *mpCopyAction;
-  QAction *mpPasteAction;
   // View Menu
   QAction *mpShowGridLinesAction;
   QAction *mpResetZoomAction;
@@ -263,6 +348,7 @@ private:
   QAction *mpCascadeWindowsAction;
   QAction *mpTileWindowsHorizontallyAction;
   QAction *mpTileWindowsVerticallyAction;
+  QAction *mpToggleTabOrSubWindowView;
   // Simulation Menu
   QAction *mpInstantiateModelAction;
   QAction *mpCheckModelAction;
@@ -274,23 +360,21 @@ private:
   QAction *mpSimulateWithAnimationAction;
 #endif
   QAction *mpSimulationSetupAction;
-  // FMI Menu
-  QAction *mpExportFMUAction;
-  QAction *mpImportFMUAction;
-  QAction *mpImportFMUModelDescriptionAction;
-  // Export Menu
-  QAction *mpExportXMLAction;
-  QAction *mpExportFigaroAction;
   // Debug Menu
   QAction *mpDebugConfigurationsAction;
   QAction *mpAttachDebuggerToRunningProcessAction;
+  // Git Menu
+  QAction *mpCreateGitRepositoryAction;
+  QAction *mpLogCurrentFileAction;
+  QAction *mpStageCurrentFileForCommitAction;
+  QAction *mpUnstageCurrentFileFromCommitAction;
+  QAction *mpCommitFilesAction;
+  QAction *mpRevertCommitAction;
+  QAction *mpCleanWorkingDirectoryAction;
   // Tools Menu
   QAction *mpShowOMCLoggerWidgetAction;
   QAction *mpShowOpenModelicaCommandPromptAction;
   QAction *mpShowOMCDiffWidgetAction;
-  QAction *mpExportToOMNotebookAction;
-  QAction *mpImportFromOMNotebookAction;
-  QAction *mpImportNgspiceNetlistAction;
   QAction *mpOpenWorkingDirectoryAction;
   QAction *mpOpenTerminalAction;
   QAction *mpOptionsAction;
@@ -302,6 +386,8 @@ private:
   QAction *mpModelicaDocumentationAction;
   QAction *mpModelicaByExampleAction;
   QAction *mpModelicaWebReferenceAction;
+  QAction *mpOMSimulatorUsersGuideAction;
+  QAction *mpOpenModelicaTLMSimulatorDocumentationAction;
   QAction *mpAboutOMEditAction;
   // Toolbar Actions
   // Shapes Toolbar Actions
@@ -313,6 +399,7 @@ private:
   QAction *mpTextShapeAction;
   QAction *mpBitmapShapeAction;
   QAction *mpConnectModeAction;
+  QAction *mpTransitionModeAction;
   // Model Switcher Toolbar Actions
   QAction *mpModelSwitcherActions[MaxRecentFiles];
   // Plot Toolbar Actions
@@ -320,19 +407,30 @@ private:
   QAction *mpReSimulateSetupAction;
   QAction *mpNewPlotWindowAction;
   QAction *mpNewParametricPlotWindowAction;
+  QAction *mpNewArrayPlotWindowAction;
+  QAction *mpNewArrayParametricPlotWindowAction;
 #if !defined(WITHOUT_OSG)
   QAction *mpNewAnimationWindowAction;
 #endif
+  QAction *mpDiagramWindowAction;
   QAction *mpClearPlotWindowAction;
   QAction *mpExportVariablesAction;
-  // Other Actions
-  QAction *mpExportAsImageAction;
-  QAction *mpExportToClipboardAction;
   // TLM Simulation Action
   QAction *mpSimulationParamsAction;
   QAction *mpFetchInterfaceDataAction;
   QAction *mpAlignInterfacesAction;
   QAction *mpTLMCoSimulationAction;
+  // OMSimulator Actions
+  QAction *mpAddSystemAction;
+  QAction *mpAddOrEditIconAction;
+  QAction *mpDeleteIconAction;
+  QAction *mpAddConnectorAction;
+  QAction *mpAddBusAction;
+  QAction *mpAddTLMBusAction;
+  QAction *mpAddSubModelAction;
+  QAction *mpOMSInstantiateModelAction;
+  QAction *mpOMSSimulateAction;
+  QAction *mpOMSArchivedSimulationsAction;
   // Toolbars
   QMenu *mpRecentFilesMenu;
   QMenu *mpLibrariesMenu;
@@ -341,28 +439,41 @@ private:
   QToolBar *mpViewToolBar;
   QToolBar *mpShapesToolBar;
   QToolBar *mpModelSwitcherToolBar;
-  QToolButton *mpModelSwitcherToolButton;
   QMenu *mpModelSwitcherMenu;
+  QToolButton *mpModelSwitcherToolButton;
   QToolBar *mpCheckToolBar;
   QToolBar *mpSimulationToolBar;
   QToolBar *mpReSimulationToolBar;
   QToolBar *mpPlotToolBar;
+  QToolBar *mpDebuggerToolBar;
+  QMenu *mpDebugConfigurationMenu;
+  QToolButton *mpDebugConfigurationToolButton;
   QToolBar *mpTLMSimulationToolbar;
+  QToolBar *mpOMSimulatorToobar;
   QHash<QString, TransformationsWidget*> mTransformationsWidgetHash;
 public slots:
+  void showMessagesBrowser();
+  void switchToWelcomePerspectiveSlot();
+  void switchToModelingPerspectiveSlot();
+  void switchToPlottingPerspectiveSlot();
+  void switchToAlgorithmicDebuggingPerspectiveSlot();
+  void showSearchBrowser();
   void createNewModelicaClass();
   void openModelicaFile();
   void showOpenModelicaFileDialog();
   void loadModelicaLibrary();
+  void loadEncryptedLibrary();
   void showOpenResultFileDialog();
   void showOpenTransformationFileDialog();
-  void createNewMetaModelFile();
-  void openMetaModelFile();
+  void createNewCompositeModelFile();
+  void openCompositeModelFile();
   void loadExternalModels();
+  void createNewOMSModel();
+  void openOMSModelFile();
   void openDirectory();
   void loadSystemLibrary();
-  void readOutputFile(qint64 bytes);
-  void readErrorFile(qint64 bytes);
+  void writeOutputFileData(QString data);
+  void writeErrorFileData(QString data);
   void openRecentFile();
   void clearRecentFilesList();
   void undo();
@@ -378,6 +489,7 @@ public slots:
   void cascadeSubWindows();
   void tileSubWindowsHorizontally();
   void tileSubWindowsVertically();
+  void toggleTabOrSubWindowView();
   void instantiateModel();
   void checkModel();
   void checkAllModels();
@@ -389,6 +501,8 @@ public slots:
   void exportModelFMU();
   void importModelFMU();
   void importFMUModelDescription();
+  void exportEncryptedPackage();
+  void exportReadonlyPackage();
   void exportModelXML();
   void exportModelFigaro();
   void showOpenModelicaCommandPrompt();
@@ -399,6 +513,9 @@ public slots:
   void exportToClipboard();
   void fetchInterfaceData();
   void TLMSimulate();
+  void instantiateOMSModel(bool checked);
+  void simulateOMSModel();
+  void showOMSArchivedSimulations();
   void openWorkingDirectory();
   void openTerminal();
   void openConfigurationOptions();
@@ -410,23 +527,31 @@ public slots:
   void openModelicaDocumentation();
   void openModelicaByExample();
   void openModelicaWebReference();
+  void openOMSimulatorUsersGuide();
+  void openOpenModelicaTLMSimulatorDocumentation();
   void openAboutOMEdit();
   void toggleShapesButton();
   void openRecentModelWidget();
   void updateModelSwitcherMenu(QMdiSubWindow *pSubWindow);
+  void runDebugConfiguration();
+  void updateDebuggerToolBarMenu();
   void toggleAutoSave();
   void readInterfaceData(LibraryTreeItem *pLibraryTreeItem);
   void enableReSimulationToolbar(bool visible);
 private slots:
   void perspectiveTabChanged(int tabIndex);
   void documentationDockWidgetVisibilityChanged(bool visible);
+  void threeDViewerDockWidgetVisibilityChanged(bool visible);
   void autoSave();
-  void switchToWelcomePerspectiveSlot();
-  void switchToModelingPerspectiveSlot();
-  void switchToPlottingPerspectiveSlot();
-  void switchToAlgorithmicDebuggingPerspectiveSlot();
-  void showConfigureDialog();
+  void showDebugConfigurationsDialog();
   void showAttachToProcessDialog();
+  void createGitRepository();
+  void logCurrentFile();
+  void stageCurrentFileForCommit();
+  void unstageCurrentFileFromCommit();
+  void commitFiles();
+  void revertCommit();
+  void cleanWorkingDirectory();
 private:
   void createActions();
   void createToolbars();
@@ -438,11 +563,11 @@ private:
   void switchToAlgorithmicDebuggingPerspective();
   void closeAllWindowsButThis(QMdiArea *pMdiArea);
   void tileSubWindows(QMdiArea *pMdiArea, bool horizontally);
-  void fetchInterfaceDataHelper(LibraryTreeItem *pLibraryTreeItem);
+  void fetchInterfaceDataHelper(LibraryTreeItem *pLibraryTreeItem, QString singleModel=QString());
 protected:
-  virtual void dragEnterEvent(QDragEnterEvent *event);
-  virtual void dragMoveEvent(QDragMoveEvent *event);
-  virtual void dropEvent(QDropEvent *event);
+  virtual void dragEnterEvent(QDragEnterEvent *event) override;
+  virtual void dragMoveEvent(QDragMoveEvent *event) override;
+  virtual void dropEvent(QDropEvent *event) override;
 };
 
 class AboutOMEditDialog : public QDialog
