@@ -1,7 +1,7 @@
 /*
  * This file is part of OpenModelica.
  *
- * Copyright (c) 1998-2014, Open Source Modelica Consortium (OSMC),
+ * Copyright (c) 1998-CurrentYear, Open Source Modelica Consortium (OSMC),
  * c/o Linköpings universitet, Department of Computer and Information Science,
  * SE-58183 Linköping, Sweden.
  *
@@ -36,10 +36,9 @@
 #define OMCPROXY_H
 
 #include "OpenModelicaScriptingAPIQt.h"
-#include "Component/Component.h"
 #include "Util/StringHandler.h"
-#include "Util/Utilities.h"
 #include "Util/Helper.h"
+#include "Util/Utilities.h"
 
 class CustomExpressionBox;
 class ComponentInfo;
@@ -63,7 +62,6 @@ private:
   CustomExpressionBox *mpExpressionTextBox;
   QPushButton *mpOMCLoggerSendButton;
   QPlainTextEdit *mpOMCLoggerTextBox;
-  QTextCursor mOMCLoggerTextCursor;
   QWidget *mpOMCDiffWidget;
   Label *mpOMCDiffBeforeLabel;
   QPlainTextEdit *mpOMCDiffBeforeTextBox;
@@ -82,13 +80,14 @@ private:
   OMCInterface *mpOMCInterface;
   bool mIsLoggingEnabled;
 public:
-  OMCProxy(QWidget *pParent = 0);
+  OMCProxy(threadData_t *threadData, QWidget *pParent = 0);
   ~OMCProxy();
+  bool eventFilter(QObject *pObject, QEvent *pEvent);
   void getPreviousCommand();
   void getNextCommand();
-  bool initializeOMC();
+  bool initializeOMC(threadData_t *threadData);
   void quitOMC();
-  void sendCommand(const QString expression);
+  void sendCommand(const QString expression, bool saveToHistory = false);
   void setResult(QString value);
   QString getResult();
   void exitApplication();
@@ -123,7 +122,7 @@ public:
   bool isPartial(QString className);
   bool isReplaceable(QString parentClassName, QString className);
   StringHandler::ModelicaClasses getClassRestriction(QString className);
-  QString getParameterValue(QString className, QString parameter);
+  QString getParameterValue(const QString &className, const QString &parameter);
   QStringList getComponentModifierNames(QString className, QString name);
   QString getComponentModifierValue(QString className, QString name);
   bool setComponentModifierValue(QString className, QString name, QString modifierValue);
@@ -144,6 +143,8 @@ public:
   int getInheritanceCount(QString className);
   QString getNthInheritedClass(QString className, int num);
   QList<QString> getInheritedClasses(QString className);
+  QString getNthInheritedClassIconMapAnnotation(QString className, int num);
+  QString getNthInheritedClassDiagramMapAnnotation(QString className, int num);
   QList<ComponentInfo*> getComponents(QString className);
   QStringList getComponentAnnotations(QString className);
   QString getDocumentationAnnotationInfoHeader(LibraryTreeItem *pLibraryTreeItem, QString infoHeader);
@@ -168,7 +169,7 @@ public:
   bool saveModifiedModel(QString modelText);
   bool saveTotalModel(QString fileName, QString className);
   QString list(QString className);
-  QString listFile(QString className);
+  QString listFile(QString className, bool nestedClasses = true);
   QString diffModelicaFileListings(QString before, QString after);
   QString instantiateModel(QString className);
   bool addClassAnnotation(QString className, QString annotation);
@@ -179,7 +180,8 @@ public:
   bool renameComponent(QString className, QString oldName, QString newName);
   bool updateComponent(QString name, QString className, QString componentName, QString placementAnnotation);
   bool renameComponentInClass(QString className, QString oldName, QString newName);
-  bool updateConnection(QString from, QString to, QString className, QString annotation);
+  bool updateConnection(QString className, QString from, QString to, QString annotation);
+  bool updateConnectionNames(QString className, QString from, QString to, QString fromNew, QString toNew);
   bool setComponentProperties(QString className, QString componentName, QString isFinal, QString isFlow, QString isProtected,
                               QString isReplaceAble, QString variability, QString isInner, QString isOuter, QString causality);
   bool setComponentComment(QString className, QString componentName, QString comment);
@@ -199,6 +201,7 @@ public:
   bool simulate(QString className, QString simualtionParameters);
   bool buildModel(QString className, QString simualtionParameters);
   bool translateModel(QString className, QString simualtionParameters);
+  int readSimulationResultSize(QString fileName);
   QStringList readSimulationResultVars(QString fileName);
   bool closeSimulationResultFile();
   QString checkModel(QString className);
@@ -206,7 +209,7 @@ public:
   QString checkAllModelsRecursive(QString className);
   bool isExperiment(QString className);
   OMCInterface::getSimulationOptions_res getSimulationOptions(QString className, double defaultTolerance = 1e-6);
-  QString buildModelFMU(QString className, double version, QString type, QString fileNamePrefix, QList<QString> platforms, bool includeResources = true);
+  QString buildModelFMU(QString className, QString version, QString type, QString fileNamePrefix, QList<QString> platforms, bool includeResources = true);
   QString translateModelXML(QString className);
   QString importFMU(QString fmuName, QString outputDirectory, int logLevel, bool debugLogging, bool generateInputConnectors, bool generateOutputConnectors);
   QString importFMUModelDescription(QString fmuModelDescriptionName, QString outputDirectory, int logLevel, bool debugLogging, bool generateInputConnectors, bool generateOutputConnectors);
@@ -216,12 +219,16 @@ public:
   QString getIndexReductionMethod();
   OMCInterface::getAvailableIndexReductionMethods_res getAvailableIndexReductionMethods();
   bool setIndexReductionMethod(QString method);
+  QList<QString> getCommandLineOptions();
   bool setCommandLineOptions(QString options);
   bool clearCommandLineOptions();
+  bool enableNewInstantiation();
+  bool disableNewInstantiation();
   QString makeDocumentationUriToFileName(QString documentation);
   QString uriToFilename(QString uri);
   QString getModelicaPath();
   QStringList getAvailableLibraries();
+  QStringList getDerivedClassModifierNames(QString className);
   QString getDerivedClassModifierValue(QString className, QString modifierName);
   OMCInterface::convertUnits_res convertUnits(QString from, QString to);
   QList<QString> getDerivedUnits(QString baseUnit);
@@ -235,6 +242,10 @@ public:
   int numProcessors();
   QString help(QString topic);
   OMCInterface::getConfigFlagValidOptions_res getConfigFlagValidOptions(QString topic);
+  QString getCompiler();
+  bool setCompiler(QString compiler);
+  QString getCXXCompiler();
+  bool setCXXCompiler(QString compiler);
   bool setDebugFlags(QString flags);
   bool exportToFigaro(QString className, QString directory, QString database, QString mode, QString options, QString processor);
   bool copyClass(QString className, QString newClassName, QString withIn);
@@ -252,11 +263,15 @@ public:
   bool inferBindings(QString className);
   bool generateVerificationScenarios(QString className);
   QList<QList<QString > > getUses(QString className);
+  bool buildEncryptedPackage(QString className, bool encrypt = true);
+  QList<QString> parseEncryptedPackage(QString fileName, QString workingDirectory);
+  bool loadEncryptedPackage(QString fileName, QString workingDirectory);
 signals:
   void commandFinished();
 public slots:
-  void logCommand(QString command, QTime *commandTime);
-  void logResponse(QString response, QTime *responseTime);
+  void logCommand(QString command, QTime *commandTime) { logCommand(command, commandTime, false); }
+  void logCommand(QString command, QTime *commandTime, bool saveToHistory);
+  void logResponse(QString command, QString response, QTime *responseTime);
   void showException(QString exception);
   void openOMCLoggerWidget();
   void sendCustomExpression();
@@ -269,7 +284,7 @@ public:
   CustomExpressionBox(OMCProxy *pOMCProxy);
   OMCProxy *mpOMCProxy;
 protected:
-  virtual void keyPressEvent(QKeyEvent *event);
+  virtual void keyPressEvent(QKeyEvent *event) override;
 };
 
 #endif // OMCPROXY_H
